package semorg.gui.list;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Shell;

import semorg.gui.LecturerWindow;
import semorg.gui.MainWindow;
import semorg.gui.provider.LecturerTableProvider;
import semorg.gui.util.AssociationTabControl;
import semorg.gui.util.ForeignKeyConstraintSelector;
import semorg.gui.util.Messages;
import semorg.sql.tables.Lecturer;
import semorg.sql.tables.SimpleIDKey;
import semorg.sql.util.DBConstraint;
import semorg.sql.util.DBTableChangedListener;
import semorg.sql.util.DistinctVector;
import semorg.sql.util.UtilitySQL;

/**
 * Provides the list window for lecturer data records. This class can be
 * instantiated as normal list windows using the menu the {@link MainWindow} as
 * well as popup list windows by the controls {@link AssociationTabControl} and
 * {@link ForeignKeyConstraintSelector}.
 */
public class LecturerListWindow extends ListWindow {

    /** A static instance of the actual list window. */
    public static ListWindow instance;

    /** Instantiates a new list window. */
    public LecturerListWindow() {
	super(MainWindow.getInstance().getShell(), Lecturer.getColumns());

	mainTableViewer.setLabelProvider(new LecturerTableProvider());

	shell
		.setText(Messages
			.getString("GUIText.LecturerListWindowTitleText"));

	update();
    }

    /**
     * Instantiates a new list window as a popup.
     * 
     * @param caller
     *                the {@link AssociationTabControl} instance which wants to
     *                create this list window
     */
    public LecturerListWindow(AssociationTabControl caller) {
	super(caller.getShell(), Lecturer.getColumns(), caller);

	mainTableViewer.setLabelProvider(new LecturerTableProvider());

	shell.setText(Messages
		.getString("GUIText.PopUpLecturerListWindowTitleText"));

	update();
	shell.open();
    }

    /**
     * Instantiates a new list window as a popup.
     * 
     * @param caller
     *                the {@link ForeignKeyConstraintSelector} instance which
     *                wants to create this list window
     */
    public LecturerListWindow(ForeignKeyConstraintSelector caller) {
	super(caller.getShell(), Lecturer.getColumns(), caller);

	mainTableViewer.setLabelProvider(new LecturerTableProvider());

	shell.setText(Messages
		.getString("GUIText.PopUpLecturerListWindowTitleText"));

	update();
	shell.open();
    }

    /**
     * Adds a listener to the mainTableViewer which opens a edit dialog to edit
     * the doubleclicked item.
     * 
     * @see semorg.gui.list.ListWindow#createSpecializedMainTableListener()
     */
    protected void createSpecializedMainTableListener() {
	addUpdateListener(DBTableChangedListener.TYPE_LECTURER);

	mainTableViewer.addDoubleClickListener(new IDoubleClickListener() {

	    public void doubleClick(DoubleClickEvent event) {
		StructuredSelection selection = (StructuredSelection) event
			.getSelection();
		Lecturer selectedItem = (Lecturer) selection.getFirstElement();
		new LecturerWindow(MainWindow.getInstance().getShell(),
			selectedItem, null);
	    }
	});
    }

    /**
     * Adds a listener to the mainTableViewer which inserts the doubleclicked
     * item into the calling {@link AssociationTabControl} instance.
     * 
     * @see semorg.gui.list.ListWindow#createSpecializedMainTableListener(semorg.gui.util.AssociationTabControl)
     */
    protected void createSpecializedMainTableListener(
	    final AssociationTabControl caller) {
	addUpdateListener(DBTableChangedListener.TYPE_LECTURER);

	mainTableViewer.addDoubleClickListener(new IDoubleClickListener() {

	    public void doubleClick(DoubleClickEvent event) {
		StructuredSelection selection = (StructuredSelection) event
			.getSelection();
		Lecturer selectedItem = (Lecturer) selection.getFirstElement();

		caller.addSelectedIds(new SimpleIDKey[] { selectedItem }, true);
		shell.close();
	    }
	});
    }

    /**
     * Adds a listener to the mainTableViewer which inserts the doubleclicked
     * item into the calling {@link ForeignKeyConstraintSelector} instance. *
     * 
     * @see semorg.gui.list.ListWindow#createSpecializedMainTableListener(semorg.gui.util.ForeignKeyConstraintSelector)
     */
    protected void createSpecializedMainTableListener(
	    final ForeignKeyConstraintSelector caller) {
	addUpdateListener(DBTableChangedListener.TYPE_LECTURER);

	mainTableViewer.addDoubleClickListener(new IDoubleClickListener() {

	    public void doubleClick(DoubleClickEvent event) {
		StructuredSelection selection = (StructuredSelection) event
			.getSelection();
		Lecturer selectedItem = (Lecturer) selection.getFirstElement();
		caller.setSelectedId(selectedItem.getId());
		shell.close();
	    }
	});
    }

    /** @see semorg.gui.list.ListWindow#createSpecializedToolBarListeners() */
    protected void createSpecializedToolBarListeners() {
	editItem.addSelectionListener(new SelectionListener() {

	    public void widgetSelected(SelectionEvent event) {
		IStructuredSelection selection = (IStructuredSelection) mainTableViewer
			.getSelection();
		Lecturer selectedElement = (Lecturer) selection
			.getFirstElement();
		new LecturerWindow(MainWindow.getInstance().getShell(),
			selectedElement, null);
	    }

	    public void widgetDefaultSelected(SelectionEvent arg0) {
		widgetSelected(arg0);
	    }
	});

	deleteItem.addSelectionListener(new SelectionListener() {

	    public void widgetSelected(SelectionEvent event) {

		IStructuredSelection selection = (IStructuredSelection) mainTableViewer
			.getSelection();
		DistinctVector<Lecturer> selectedElements = new DistinctVector(
			(List<Lecturer>) selection.toList());

		String question = (selectedElements.size() > 1) ? Messages
			.getString("GUIText.DeleteLecturerItemsMessageDialogText")
			: Messages
				.getString("GUIText.DeleteLecturerItemMessageDialogText");
		MessageDialog dlg = new MessageDialog(
			MainWindow.getInstance().getShell(),
			Messages
				.getString("GUIText.DeleteItemMessageDialogTitle"),
			null,
			question,
			MessageDialog.QUESTION,
			new String[] {
				Messages
					.getString("GUIText.DeleteItemMessageDialogYesButtonText"),
				Messages
					.getString("GUIText.DeleteItemMessageDialogNoButtonText") },
			0);

		if (dlg.open() != 0) {// not "yes" selected
		    return;
		}

		try {
		    Iterator<Lecturer> it = selectedElements.iterator();
		    while (it.hasNext()) {
			int id = it.next().getId();
			if (Lecturer.lockedIds.contains(new Integer(id))) {
			    MessageDialog
				    .openInformation(
					    shell,
					    Messages
						    .getString("GUIText.DeleteErrorDialogTitle"),
					    Messages
						    .getString("GUIText.DeleteErrorDialogText"));
			    return;
			}
		    }
		    Lecturer.removeFromDB(selectedElements);
		    LecturerListWindow.instance.update();
		} catch (SQLException e) {
		    e.printStackTrace();
		}
	    }

	    public void widgetDefaultSelected(SelectionEvent arg0) {
		widgetSelected(arg0);
	    }
	});

	newItem.addSelectionListener(new SelectionListener() {

	    public void widgetSelected(SelectionEvent event) {
		new LecturerWindow(MainWindow.getInstance().getShell(), null,
			null);
	    }

	    public void widgetDefaultSelected(SelectionEvent event) {
		widgetSelected(event);
	    }
	});
    }

    /** @see semorg.gui.list.ListWindow#createChooseButtonListener(semorg.gui.util.AssociationTabControl) */
    protected void createChooseButtonListener(final AssociationTabControl caller) {
	chooseButton.addSelectionListener(new SelectionAdapter() {
	    public void widgetSelected(SelectionEvent e) {
		StructuredSelection selection = (StructuredSelection) mainTableViewer
			.getSelection();
		if (!selection.isEmpty()) {

		    Object[] selectedItems = selection.toArray();
		    SimpleIDKey[] selectedElements = (SimpleIDKey[]) UtilitySQL
			    .castArray(selectedItems, SimpleIDKey.class);

		    caller.addSelectedIds(selectedElements, true);
		    shell.close();
		}
	    }
	});
    }

    /**
     * Adds a listener to the mainTableViewer which inserts the doubleclicked
     * item into the calling {@link ForeignKeyConstraintSelector} instance.
     * 
     * @see semorg.gui.list.ListWindow#createSpecializedMainTableListener(semorg.gui.util.ForeignKeyConstraintSelector)
     */
    protected void createChooseButtonListener(
	    final ForeignKeyConstraintSelector caller) {
	chooseButton.addSelectionListener(new SelectionAdapter() {
	    public void widgetSelected(SelectionEvent e) {
		StructuredSelection selection = (StructuredSelection) mainTableViewer
			.getSelection();
		if (!selection.isEmpty()) {
		    Lecturer selectedItem = (Lecturer) selection
			    .getFirstElement();
		    caller.setSelectedId(selectedItem.getId());
		    shell.close();
		}
	    }
	});
    }

    /** Opens the shell of the static list window instance. */
    public static void openList() {
	instance.shell.open();
    }

    /** @see semorg.gui.list.ListWindow#getTableInputFromDB() */
    protected Object getTableInputFromDB() {
	Vector<DBConstraint> constraints = null;

	// if active, read filter
	if (filterItem.getSelection()) {
	    try {
		constraints = filter.getConstraints();
	    } catch (Exception e) {
		// invalid filter value => switch off the filter
		filterItem.setSelection(false);
	    }
	}

	try {
	    return Lecturer.getVectorFromDB(constraints, sortString);
	} catch (SQLException e) {
	    e.printStackTrace();
	}
	return new DistinctVector<Lecturer>();
    }

    /**
     * Creates the static list window instance.
     * 
     * @param parent
     *                the parent shell.
     */
    public static void createWindow(Shell parent) {
	instance = new LecturerListWindow();
    }

}
