package semorg.gui.util;

import java.io.InputStream;

import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;

import semorg.sql.util.UtilitySQL;

/**
 * Utility class which encapsulates some useful methods for the GUI layer.
 */
public class UtilityGUI {
    /** A registry containing all {@link Image} objects of the application. */
    private static ImageRegistry imgReg;

    /**
     * Needed for regression tests with ATOSj. (for identifying the GUI
     * elements).
     */
    public static final String ATOSJ_COMPONENT_NAME_KEY = "ATOSJ_COMPONENT_NAME_KEY";
    /**
     * {@link Color} instance for red-coloring of text fields (unfilled
     * mandatory fields).
     */
    public static final Color LIGHT_YELLOW = new Color(Display.getDefault(),
	    255, 255, 153);
    /**
     * {@link Color} instance for yellow-coloring of text fields (mandatory
     * fields).
     */
    public static final Color LIGHT_RED = new Color(Display.getDefault(), 255,
	    192, 192);
    /**
     * Colors empty mandatory fields with red color if they loose the focus. The
     * field get colored yellow if they regain the focus.<br>
     * <br>
     * 
     * Thus this listener marks input errors in the GUI.
     */
    public static FocusListener checkEmptyListener = new FocusListener() {

	public void focusGained(FocusEvent e) {
	    if (e.getSource() instanceof Text) {
		Text t = (Text) e.getSource();
		t.setBackground(LIGHT_YELLOW);
	    } else if (e.getSource() instanceof Combo) {
		Combo c = (Combo) e.getSource();
		c.setBackground(LIGHT_YELLOW);
	    }

	}

	public void focusLost(FocusEvent e) {
	    if (e.getSource() instanceof Text) {
		Text t = (Text) e.getSource();
		if (t.getText().trim().length() == 0)
		    t.setBackground(LIGHT_RED);
	    } else if (e.getSource() instanceof Combo) {
		Combo c = (Combo) e.getSource();
		if (c.getText().trim().length() == 0)
		    c.setBackground(LIGHT_RED);
	    }

	}
    };

    /**
     * Checks if the entered String is a valid integer. If not it colors the
     * text field red. Otherwise the field gets the color white.
     */
    public static FocusListener checkCorrectIntListener = new FocusListener() {

	public void focusGained(FocusEvent e) {
	    if (e.getSource() instanceof Text) {
		Text t = (Text) e.getSource();
		t.setBackground(Display.getDefault().getSystemColor(
			SWT.COLOR_WHITE));
	    }
	}

	public void focusLost(FocusEvent e) {
	    if (e.getSource() instanceof Text) {
		Text t = (Text) e.getSource();
		String text = t.getText();
		if (!UtilitySQL.isCorrectInteger(text))
		    t.setBackground(LIGHT_RED);
	    }
	}
    };

    /**
     * Checks if the entered String is a valid float number. If not it colors
     * the text field red. Otherwise the field gets the color white.
     */
    public static FocusListener checkCorrectFloatListener = new FocusListener() {

	public void focusGained(FocusEvent e) {
	    if (e.getSource() instanceof Text) {
		Text t = (Text) e.getSource();
		t.setBackground(Display.getDefault().getSystemColor(
			SWT.COLOR_WHITE));
	    }
	}

	public void focusLost(FocusEvent e) {
	    if (e.getSource() instanceof Text) {
		Text t = (Text) e.getSource();
		String text = t.getText();
		if (!UtilitySQL.isCorrectFloat(text))
		    t.setBackground(LIGHT_RED);
	    }
	}
    };

    /**
     * Creates for a SWT control instance a {@link FormData} instance and
     * returns it.<br>
     * <br>
     * 
     * The {@link FormData} object specifies how the {@link FormLayout} manager
     * should adjust the widget.The {@link FormData} instance itselfs is
     * specified by {@link FormAttachment}s, which attach the four borders
     * (top, bottom, left, right) of a widget to several positions of shell.<br>
     * <br>
     * 
     * <i>Hint:</i>
     * <ul>
     * <li> If you use the {@link FormLayout}, you MUST assign to every widget
     * a OWN {@link FormData}. </li>
     * </ul>
     * 
     * @param topPercentage
     *                Position used to attach the top border of a widget.
     * @param topOffset
     *                Offset to the position of the top border.
     * @param bottomPercentage
     *                Position used to attach the bottom border of a widget.
     * @param bottomOffset
     *                Offset to the position of the bottom border.
     * @param leftPercentage
     *                Position used to attach the left border of a widget.
     * @param leftOffset
     *                Offset to the position of the left border.
     * @param rightPercentage
     *                Position used to attach the right border of a widget.
     * @param rightOffset
     *                Offset to the position of the left border.
     * @return {@link FormData} object created with the given parameters.
     */
    public static FormData getFormData(Object topPercentage, int topOffset,
	    Object bottomPercentage, int bottomOffset, Object leftPercentage,
	    int leftOffset, Object rightPercentage, int rightOffset) {
	FormData fm = new FormData();

	if (topPercentage instanceof Control)
	    fm.top = new FormAttachment((Control) topPercentage, topOffset);
	else if (topPercentage instanceof Integer
		&& UtilityGUI.isvalidPercentage((Integer) topPercentage))
	    fm.top = new FormAttachment((Integer) topPercentage, topOffset);

	if (bottomPercentage instanceof Control)
	    fm.bottom = new FormAttachment((Control) bottomPercentage,
		    bottomOffset);
	else if (bottomPercentage instanceof Integer
		&& UtilityGUI.isvalidPercentage((Integer) bottomPercentage))
	    fm.bottom = new FormAttachment((Integer) bottomPercentage,
		    bottomOffset);

	if (leftPercentage instanceof Control)
	    fm.left = new FormAttachment((Control) leftPercentage, leftOffset);
	else if (leftPercentage instanceof Integer
		&& UtilityGUI.isvalidPercentage((Integer) leftPercentage))
	    fm.left = new FormAttachment((Integer) leftPercentage, leftOffset);

	if (rightPercentage instanceof Control)
	    fm.right = new FormAttachment((Control) rightPercentage,
		    rightOffset);
	else if (rightPercentage instanceof Integer
		&& UtilityGUI.isvalidPercentage((Integer) rightPercentage))
	    fm.right = new FormAttachment((Integer) rightPercentage,
		    rightOffset);

	return fm;
    }

    /**
     * Checks if the given percentage is valid.
     * 
     * @param val
     *                value to check.
     * @return <tt>true</tt> if the given value is a valid percentage,
     *         <tt>false</tt> otherwise.
     */
    private static boolean isvalidPercentage(int val) {
	return ((val >= 0) && (val <= 100));
    }

    /**
     * Creates a tab whose contents is a {@link Label} and a {@link Text} field.
     * 
     * @param parent
     *                TabFolder, which contains the tab.
     * @param name
     *                name of the tab.
     * @param font
     *                Desired font. <code>null</code>, if using the default
     *                font.
     * @param limit
     *                size of the {@link Text} field.
     * @return created {@link Text} field.
     */
    public static Text createTextTab(TabFolder parent, String name, Font font,
	    int limit) {
	TabItem textTab = new TabItem(parent, SWT.NULL);
	textTab.setText(name);

	Composite tabContent = new Composite(parent, SWT.NONE);
	textTab.setControl(tabContent);
	tabContent.setLayout(new FormLayout());

	Label header = new Label(tabContent, SWT.READ_ONLY);
	header.setText(name);
	header.setLayoutData(getFormData(14, 0, null, 0, 0, 9, null, 0));

	Text text = new Text(tabContent, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL
		| SWT.WRAP);
	text.setTextLimit(limit);
	text.setLayoutData(getFormData(0, 39, 100, -16, 0, 9, 100, -10));

	if (font != null) {
	    header.setFont(font);
	    text.setFont(font);
	}
	return text;
    }

    /**
     * Sets the number of characters a {@link Text} field can take as well as
     * the width of this {@link Text} field.
     * 
     * @param comp
     *                {@link Text} field, whose length should get modified.
     * @param size
     *                Number of characters which can be entered. The width is
     *                set to <code>6*size</code>.
     */
    public static void setLength(Text comp, int size) {
	GridData gd; // um die Pixel-Breite des Textfelds festzulegen
	Object ld = comp.getLayoutData();
	if (ld != null && ld instanceof GridData)
	    gd = (GridData) ld;
	else
	    gd = new GridData();

	comp.setTextLimit(size);
	gd.widthHint = 6 * size;
	comp.setLayoutData(gd);
    }

    /**
     * Sets the number of characters a {@link Text} field can take as well as
     * the number of characters shown.
     * 
     * @param comp
     *                {@link Text} field, whose length should get modified.
     * @param textLength
     *                Number of characters which can be entered.
     * @param lettersShown
     *                Number of characters shown.
     */
    public static void setLength(Text comp, int textLength, int lettersShown) {
	GridData gd;
	Object ld = comp.getLayoutData();
	if (ld != null && ld instanceof GridData)
	    gd = (GridData) ld;
	else
	    gd = new GridData();

	comp.setTextLimit(textLength);
	gd.widthHint = 6 * lettersShown;
	comp.setLayoutData(gd);
    }

    /**
     * Sets the number of characters a {@link Combo} field can take as well as
     * the width of this {@link Combo} field.
     * 
     * @param comp
     *                {@link Combo} field, whose length should get modified.
     * @param size
     *                Number of characters which can be entered. The width is
     *                set to <code>6*size</code>.
     */
    public static void setLength(Combo comp, int size) {
	GridData gd;
	Object ld = comp.getLayoutData();
	if (ld != null && ld instanceof GridData)
	    gd = (GridData) ld;
	else
	    gd = new GridData();

	comp.setTextLimit(size);
	gd.widthHint = 6 * size;
	comp.setLayoutData(gd);
    }

    /**
     * Aligns a {@link Label} right.
     * 
     * @param label
     *                the label to align.
     */
    public static void alignRight(Control label) {
	GridData gd;
	Object ld = label.getLayoutData();
	if (ld != null && ld instanceof GridData)
	    gd = (GridData) ld;
	else
	    gd = new GridData();
	gd.horizontalAlignment = GridData.END;
	label.setLayoutData(gd);
    }

    /**
     * Returns for a given image name a {@link Image} object from the
     * {@link ImageRegistry} {@link UtilityGUI#imgReg}. If the image is not
     * already present in the registry it gets added.
     * 
     * @param name
     *                (file-)name of the image.
     */
    public static Image getImage(String name) {
	if (UtilityGUI.imgReg == null)
	    UtilityGUI.imgReg = new ImageRegistry(Display.getDefault());

	Image img = UtilityGUI.imgReg.get(name);
	if (img == null) {
	    UtilityGUI.initImage(name);
	    return UtilityGUI.imgReg.get(name);
	} else {
	    return img;
	}
    }

    /**
     * Loads a image into a {@link ImageRegistry}.
     * 
     * @param name
     *                (file-)name of the image to add.
     */
    private static void initImage(String name) {
	InputStream stream = UtilitySQL.class.getClassLoader()
		.getResourceAsStream("semorg/gui/icon/" + name);
	UtilityGUI.imgReg.put(name, new Image(Display.getDefault(), stream));
    }

    /**
     * Disposes all images by disposing the {@link ImageRegistry}.
     */
    public static void disposeImages() {
	UtilityGUI.imgReg.dispose();
    }

}
