package semorg.sql.tables;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import semorg.gui.util.Messages;
import semorg.sql.access.DBAccess;
import semorg.sql.util.DBColumn;
import semorg.sql.util.DBTableChangedListener;
import semorg.sql.util.KeyPair;
import semorg.sql.util.UtilitySQL;

/**
 * Models the association "Conduct" between the classes Lecturer and
 * Presentation: a Lecturer conducts a SeminarType (association has a *
 * cardinality).
 */
public class Conduct extends AbstractTable {

    /**
     * A list of id-pairs of locked records. This list is meant to prevent the
     * manipulation / deletion of records whose association is currently edited.
     */
    public static List<KeyPair> lockedIds = new ArrayList<KeyPair>();

    /** The id of the lecturer. */
    protected int lecturerId;

    /** The id of the seminartype. */
    protected int seminartypeId;

    /** The name of the corresponding table in the database. */
    private static String tableName = "conduct";

    /** SQL query to insert a conduct record. */
    protected static String conductInsertString = "INSERT INTO " + tableName
	    + " VALUES(?, ?, ?, ?)";

    /** SQL query to delete a conduct record. */
    protected static String conductDeleteString = "DELETE FROM " + tableName
	    + " WHERE lecturerid=? AND seminartypeid=?";

    /** SQL statement for creating the table "conduct" (if not exists). */
    private static String createTableSQLString = "CREATE TABLE IF NOT EXISTS `"
	    + tableName
	    + "` ("
	    + "`lecturerid` integer NOT NULL,"
	    + "`seminartypeid` integer NOT NULL,"
	    + "`creationdate` timestamp NOT NULL default '0000-00-00 00:00:00',"
	    + "`modificationdate` timestamp NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,"
	    + "PRIMARY KEY  (`lecturerid`, `seminartypeid`),"
	    + "CONSTRAINT `cond_lecturer_fk` FOREIGN KEY (`lecturerid`) REFERENCES `lecturer` (`person_fk`) ON DELETE CASCADE, "
	    + "CONSTRAINT `cond_semtype_fk` FOREIGN KEY (`seminartypeid`) REFERENCES `seminartype` (`id_pk`) ON DELETE CASCADE)";

    /** Creates an instance of the class Conduct with the given parameters. */
    public Conduct(int lecturerId, int seminartypeId, Timestamp creationDate,
	    Timestamp modificationDate) {

	super(creationDate, modificationDate);

	this.lecturerId = lecturerId;
	this.seminartypeId = seminartypeId;
    }

    /**
     * Checks if the according DB table exists.
     * 
     * @see semorg.sql.tables.AbstractTable#tableOK(String)
     * @return <code>true</code>, if the table is ok, <code>false</code>
     *         otherwise.
     */
    public static boolean tableOK() {
	return AbstractTable.tableOK(tableName);
    }

    /**
     * Creates the table "conduct" if it does not exist.
     * 
     * @param statement
     *                instance of the class Statement, which is used for
     *                executing the SQL statement {@link #createTableSQLString}.
     * @throws SQLException
     *                 If the execution or closing of the given statement fails.
     */
    public static void createConductTable(Statement statement)
	    throws SQLException {
	statement.execute(createTableSQLString);
	statement.close();
    }

    /**
     * Inserts a conduct record into the db using the
     * {@link #conductInsertString}. Finally it fires a
     * TableChangedEvent-Listener with the appropriate type.
     * 
     * @see semorg.sql.util.DBTableChangedListener - overview of all listener
     *      types
     * @throws SQLException
     *                 If the creation, manipulation, execution or closing of a
     *                 PreparedStatement instance used for querying the database
     *                 fails.
     */
    public void insertIntoDB() throws SQLException {
	PreparedStatement insertStmt = DBAccess.dbAccess
		.getPreparedStatement(conductInsertString);

	insertStmt.setInt(1, lecturerId);
	insertStmt.setInt(2, seminartypeId);

	insertStmt.setTimestamp(3, UtilitySQL.convertToTimestamp(creationDate));
	insertStmt
		.setTimestamp(4, UtilitySQL.convertToTimestamp(modificationDate));

	insertStmt.executeUpdate();

	insertStmt.close();

	AbstractTable
		.fireTableChangedEvent(DBTableChangedListener.TYPE_CONDUCT);
    }

    /**
     * Delets a conduct record from the db using the
     * {@link #conductDeleteString}. Finally it fires a
     * TableChangedEvent-Listener with the appropriate type.
     * 
     * @see semorg.sql.util.DBTableChangedListener - overview of all listener
     *      types
     * @throws SQLException
     *                 If the creation, manipulation, execution or closing of a
     *                 PreparedStatement instance used for querying the database
     *                 fails.
     */
    public void deleteFromDB() throws SQLException {

	PreparedStatement deleteStmt = DBAccess.dbAccess
		.getPreparedStatement(conductDeleteString);

	deleteStmt.setInt(1, lecturerId);
	deleteStmt.setInt(2, seminartypeId);

	deleteStmt.executeUpdate();

	deleteStmt.close();

	AbstractTable
		.fireTableChangedEvent(DBTableChangedListener.TYPE_CONDUCT);
    }

    /**
     * Returns a vector of {@link semorg.sql.util.DBColumn} instances according to
     * the properties of the table "conduct".
     * 
     * @return the vector of DBColumn instances according to the properties of
     *         the class Conduct.
     * @see semorg.sql.tables.AbstractTable#getColumns(String tableAbbreviation)
     */
    public static Vector<DBColumn> getColumns() {
	Vector<DBColumn> columns = new Vector<DBColumn>();
	columns.add(new DBColumn("lecturerid", Messages
		.getString("MAP.Conduct.LecturerId"), Integer.class));
	columns.add(new DBColumn("seminartypeid", Messages
		.getString("MAP.Conduct.SeminarTypeId"), Integer.class));
	columns.addAll(AbstractTable.getColumns(""));
	return columns;

    }

    public int getLecturerId() {
	return lecturerId;
    }

    public int getSeminartypeId() {
	return seminartypeId;
    }
}
