package semorg.sql.util;

import java.lang.reflect.Array;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;




/**
 * UtilitySQL-Class which encapsulates some useful methods for the database
 * layer.
 */
public class UtilitySQL {

    /** Date formatter using standard localization. */
    public static final DateFormat dateOnlyFormatter = DateFormat
	    .getDateInstance();
    /** Date and time formatter using standard localization. */
    public static final DateFormat dateAndTimeFormatter = DateFormat
	    .getDateTimeInstance();
    /** Time formatter using the format <code>HH:mm</code>. */
    public static final DateFormat timeOnlyFormatter = new SimpleDateFormat(
	    "HH:mm");

    /** Parses strings for floating-point numbers. */
    private static NumberFormat floatParser = NumberFormat.getNumberInstance();

    /** Converts a JAVA date into SQL date. */
    public static java.sql.Date convertToSQLDate(java.util.Date date) {
	return new java.sql.Date(date.getTime());
    }

    /** Converts a JAVA timestamp inti a SQL timestamp. */
    public static java.sql.Timestamp convertToTimestamp(java.util.Date date) {
	return new java.sql.Timestamp(date.getTime());
    }

    /**
     * Parses a float-number from a given string.
     * 
     * @return {@link Float} object, if the parsing was successful otherwise
     *         <code>null</code>.
     */
    public static Float parseFloat(String floatString) {
	if (floatString == null || floatString.trim().length() == 0)
	    return null;
	else
	    try {
		return new Float(floatParser.parse(floatString).floatValue());
	    } catch (Exception e) {
		return null;
	    }
    }

    /**
     * Checks whether a given string is a valid float number.
     * 
     * @return If <code>null</code>, only whitespaces, a empty String or a
     *         valid float number was given the method returns <code>true</code>,
     *         otherwise <code>false</code>.
     */
    public static boolean isCorrectFloat(String floatString) {
	if (floatString == null || floatString.trim().length() == 0)
	    return true;
	else {
	    floatString = floatString.trim();
	    // Index, bei dem ein Parse-Fehler auftrat; bei einer korrekten
	    // Float-Zahl also letzter Index + 1
	    ParsePosition pp = new ParsePosition(0);
	    floatParser.parse(floatString, pp);
	    return pp.getIndex() == floatString.length();
	}
    }

    /**
     * Parses a integer from a given string.
     * 
     * @return {@link Integer} object, if the parsing was successful otherwise
     *         <code>null</code>.
     */
    public static Integer parseInteger(String integerString) {
	if (integerString == null || integerString.trim().length() == 0)
	    return null;
	else
	    try {
		return new Integer(Integer.parseInt(integerString));
	    } catch (Exception e) {
		return null;
	    }
    }

    /**
     * Checks whether a given string is a valid integer.
     * 
     * @return If <code>null</code>, only whitespaces, a empty String or a
     *         valid integer was given the method returns <code>true</code>,
     *         otherwise <code>false</code>.
     */
    public static boolean isCorrectInteger(String integerString) {
	if (integerString == null || integerString.trim().length() == 0)
	    return true;
	else
	    try {
		Integer.parseInt(integerString);
		return true;
	    } catch (Exception e) {
		return false;
	    }
    }

    /**
     * Creates a string from a given {@link Float} object.
     * 
     * @return Empty string, if <code>null</code> was given, otherwise the
     *         created string.
     */
    public static String createFloatText(Float theFloat) {
	if (theFloat == null)
	    return "";

	return floatParser.format(theFloat);
    }

    /**
     * Sets several properties for the FloatFormater/FloatParser:
     * 
     * <ul>
     * <li>Maximal and minimal number of fraction digits.</li>
     * <li>No grouping of numbers, e.g. grouping: 1223786 -> 1.234.786</li>
     * </ul>
     */
    public static void initUtilitySQL() {
	floatParser.setMaximumFractionDigits(2);
	floatParser.setMinimumFractionDigits(2);
	floatParser.setGroupingUsed(false);
    }

    /**
     * Does a type cast of all elements of a given array.
     * 
     * @param array
     *                Array, whose elements get type casted.
     * @param clazz
     *                Desired class to cast the elements to.
     * @return Array with type casted elements.
     */
    public static Object[] castArray(Object[] array, Class<?> clazz) {

	Object[] returnArray = (Object[]) Array
		.newInstance(clazz, array.length);

	for (int i = 0; i < array.length; i++)
	    returnArray[i] = (array[i]);

	return returnArray;
    }
}
